#coding:utf-8
import _env


def pre_config(o):

    o.HOST = 'z32e1.tk'
    o.SITE_NAME   = '42区'
    o.SITE_SLOGO  = "找到给你答案的人"


    o.DEBUG = False
    o.PORT = 42001
    o.PROCESS_NUM = 4
    

    o.MYSQL_HOST = '127.0.0.1'
    o.MYSQL_PORT = '3306'
    o.MYSQL_USER = 'zweb'
    o.MYSQL_PASSWD = '42qudev'
    o.MYSQL_PREFIX = o.MYSQL_USER

    o.HOST_DEV_PREFIX = "dev-"


    o.MEMCACHED_ADDR = (
        '127.0.0.1:11211',

    )
    o.DISABLE_LOCAL_CACHED = False

    o.REDIS_CONFIG = dict(
#        unix_socket_path = "/tmp/redis.sock"
        host = 'localhost', 
        port = 6379, 
        db   = 0
    )

def post_config(o):
    o.SITE_URL = "//%s"%o.HOST
    o.SITE_HTTP = "http:%s"%o.SITE_URL

    # 注意：备份路径只为数据备份准备，而不是表备份
    o.BACKUP_PATH = '/mnt/backup/%s'%o.HOST

    # css js 静态文件的域名
    o.HOST_CSS_JS = 's.%s'%o.HOST
    o.URL_FS =  'http://%s'%o.HOST_CSS_JS




#coding:utf-8
import re

TLD_NAME = (
'ac.at',
'ac.cn',
'ac.id',
'ac.jp',
'ac.th',
'adv.br',
'ae',
'ag',
'ah.cn',
'ai',
'am',
'appspot.com',
'ar',
'art.pl',
'as',
'asia',
'at',
'au',
'ba',
'bd',
'be',
'bg',
'biz',
'bj.cn',
'bl.uk',
'blog.br',
'bo',
'br',
'br.com',
'by',
'bz',
'ca',
'cat',
'cc',
'cd',
'ch',
'cl',
'club.tw',
'cn',
'cn.com',
'co',
'co.at',
'co.id',
'co.in',
'co.jp',
'co.kr',
'com',
'com.br',
'com.cn',
'com.co',
'com.do',
'com.ec',
'com.es',
'com.hk',
'com.lb',
'com.mx',
'com.my',
'com.pe',
'com.ph',
'com.pl',
'com.pt',
'com.ru',
'com.sg',
'com.tw',
'com.ua',
'com.vn',
'coop',
'cq.cn',
'cx',
'cz',
'de',
'dk',
'donetsk.ua',
'ec',
'edu',
'edu.br',
'edu.cn',
'edu.hk',
'edu.mx',
'edu.my',
'edu.ph',
'edu.pl',
'edu.sa',
'edu.sg',
'edu.tw',
'ee',
'elk.pl',
'eng.br',
'es',
'etc.br',
'eti.br',
'eu',
'eu.int',
'fed.us',
'fi',
'fj.cn',
'fm',
'fr',
'gc.ca',
'gd',
'gd.cn',
'ge',
'gg',
'go.cr',
'go.jp',
'go.th',
'gob.mx',
'gob.pe',
'gouv.fr',
'gov',
'gov.br',
'gov.cn',
'gov.co',
'gov.hk',
'gov.mo',
'gov.sg',
'gov.tw',
'gov.uk',
'gp',
'gr',
'gr.jp',
'gs',
'gt',
'gx.cn',
'gy',
'gz.cn',
'ha.cn',
'hb.cn',
'he.cn',
'hi.cn',
'hk',
'hk.cn',
'hm',
'hn.cn',
'hr',
'hu',
'i.ph',
'idv.tw',
'ie',
'if.ua',
'iki.fi',
'il',
'im',
'in',
'in.th',
'in.ua',
'info',
'info.pl',
'int',
'int.ru',
'io',
'ir',
'is',
'it',
'je',
'jl.cn',
'jp',
'js.cn',
'jx.cn',
'k12.mi.us',
'kharkov.ua',
'kiev.ua',
'kr',
'ks.us',
'kz',
'la',
'li',
'lib.in.us',
'lk',
'ln.cn',
'local',
'lt',
'lu',
'lukow.pl',
'lv',
'lviv.ua',
'ly',
'ma',
'mat.br',
'md',
'me',
'med.br',
'mil',
'mn',
'mn.us',
'mo.cn',
'mobi',
'mp',
'ms',
'msk.ru',
'mu',
'museum',
'mx',
'my',
'naklo.pl',
'name',
'name.my',
'ne.jp',
'net',
'net.cn',
'net.my',
'net.pl',
'net.ru',
'net.tw',
'nhs.uk',
'nl',
'nm.cn',
'no',
'nom.br',
'np',
'nu',
'nz',
'odessa.ua',
'on.ca',
'or.id',
'or.it',
'or.jp',
'or.kr',
'or.th',
'org',
'org.br',
'org.cn',
'org.es',
'org.hk',
'org.in',
'org.mk',
'org.mx',
'org.pl',
'org.ru',
'org.sg',
'org.tw',
'org.ua',
'oslo.no',
'padova.it',
'pc.pl',
'pe',
'pe.kr',
'ph',
'pisz.pl',
'pk',
'pl',
'pp.ru',
'pri.ee',
'pro',
'ps',
'pt',
'py',
're',
'ro',
'rs',
'ru',
'sc',
'sc.cn',
'sch.id',
'sd.cn',
'se',
'sebastopol.ua',
'sg',
'sh',
'sh.cn',
'si',
'sk',
'sn.cn',
'srv.br',
'st',
'su',
'tc',
'te.ua',
'tk',
'tl',
'tm',
'to',
'tomsk.ru',
'tr',
'tt',
'tv',
'tw',
'tw.cn',
'uk',
'uk.com',
'us',
'us.com',
'uy',
'uz',
'va.us',
'vc',
've',
'vg',
'vic.gov.au',
'vn',
'vn.ua',
'waw.pl',
'web.id',
'wroc.pl',
'ws',
'xxx',
'za',
'za.net',
'za.org',
'zj.cn',
'zm',
)

RE_TLD_NAME = re.compile(
    r".*?((?:[^\.]+\.)(%s))$"%(
        '|'.join([i.replace('.', '\\.') for i in TLD_NAME])
    )
)

def tld_name(domain):
    m = RE_TLD_NAME.match(domain)
    if m:
        return m.groups()[0]

if __name__ == '__main__':
    print tld_name('www.work.com')
    print tld_name('zsp.com')
    print tld_name('www.zsp.com')
    print tld_name('www.zsp.edu.cn')
    print tld_name('xx.www.zsp.edu.cn')
    print tld_name('xx.www.zsp.com.cn')




#!/usr/bin/env python
#coding:utf-8

class JsDict(object):
    def __init__(self, o=None):
        if o is None:
            o = {}
        self.__dict__['_JsDict__o'] = o

    def __iter__(self):
        for i in self.__o.iteritems():
            yield i

    def __nonzero__(self):
        return bool(self.__o)

    def __setattr__(self, name, val):
        if val is not None:
            self.__o[name] = val

    def __getattr__(self, name):
        return self.__o.get(name, '')

    def __getitem__(self, name):
        return self.__o.get(name, '')

    def __contains__(self, b):
        return b in self.__o

    def __setitem__(self, name, val):
        if val is not None:
            self.__o[name] = val

    def __delitem__(self, name):
        del self.__o[name]





import sys, os, subprocess

def quote_first_command_arg( arg):
    """
    There's a bug in Windows when running an executable that's
    located inside a path with a space in it.  This method handles
    that case, or on non-Windows systems or an executable with no
    spaces, it just leaves well enough alone.
    """
    if (sys.platform!='win32'
        or ' ' not in arg):
        # Problem does not apply:
        return arg
    try:
        import win32api
    except ImportError:
        raise ValueError(
            'The executable %r contains a space, and in order to '
            'handle this issue you must have the win32api module '
            'installed' % arg)
    arg = win32api.GetShortPathName(arg)
    return arg

def _turn_sigterm_into_systemexit():
    """
    Attempts to turn a SIGTERM exception into a SystemExit exception.
    """
    try:
        import signal
    except ImportError:
        return
    def handle_term(signo, frame):
        raise SystemExit
    signal.signal(signal.SIGTERM, handle_term)

def restart_with_monitor():
    while 1:
        args = [quote_first_command_arg(sys.executable)] + sys.argv
        new_environ = os.environ.copy()
        new_environ['_run'] = '1'
        proc = None
        try:
            try:
                _turn_sigterm_into_systemexit()
                proc = subprocess.Popen(args, env=new_environ)
                exit_code = proc.wait()
                proc = None
            except KeyboardInterrupt:
                print '^C caught in monitor process'
                return 1
        finally:
            if (proc is not None
                and hasattr(os, 'kill')):
                import signal
                try:
                    os.kill(proc.pid, signal.SIGTERM)
                except (OSError, IOError):
                    pass

        # Reloader always exits with code 3; but if we are
        # a monitor, any exit code will restart
        if exit_code != 3:
            return exit_code

def _auto_reload(app):
    if os.environ.get('_run'):
        import reloader
        reloader.install()
        app()
    else:
        restart_with_monitor()

def auto_reload(app):
    from time import sleep
    while True:
        _auto_reload(app)
        print '\nAuto Reloading After 3 seconds : '
        for i in xrange(10, 0, -1):
            sleep(0.3)
            print i,
        print ''



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import Cookie
import base64
import calendar
import datetime
import email.utils
import functools
import gzip
import hashlib
import hmac
import httplib
import logging
import mimetypes
import os.path
import re
import stat
import sys
import time
import types
import urllib
import urlparse
import uuid

from tornado import web
from tornado.web import HTTPError, utf8
from zweb.lib.tld_name import tld_name
from profile_middleware import profile_middleware

from tornado import escape
from tornado import locale
from tornado import stack_context
from tornado import template

def set_cookie(self, name, value, domain=None, expires=None, path='/',
               expires_days=None, **kwargs):
    """Sets the given cookie name/value with the given options.

    Additional keyword arguments are set on the Cookie.Morsel
    directly.
    See http://docs.python.org/library/cookie.html#morsel-objects
    for available attributes.
    """
    if domain is None:
        domain = '.%s'%tld_name(self.request.host)

    self.set_header("P3P","CP=CAO PSA OUR")

    name = escape.native_str(name)
    value = escape.native_str(value)
    if re.search(r"[\x00-\x20]", name + value):
        # Don't let us accidentally inject bad stuff
        raise ValueError("Invalid cookie %r: %r" % (name, value))
    if not hasattr(self, "_new_cookie"):
        self._new_cookie = Cookie.SimpleCookie()
    if name in self._new_cookie:
        del self._new_cookie[name]
    self._new_cookie[name] = value
    morsel = self._new_cookie[name]
    if domain:
        morsel["domain"] = domain



    if expires_days is not None and not expires:
        expires = datetime.datetime.utcnow() + datetime.timedelta(
            days=expires_days)
    if expires:
        if type(expires) is not str:
            timestamp = calendar.timegm(expires.utctimetuple())
            expires = email.utils.formatdate(
                timestamp, localtime=False, usegmt=True
            )
    else:
        expires  = 'Tue, 01 Jan 2030 00:00:00 GMT'
    morsel['expires'] = expires

    if path:
        morsel["path"] = path
    for k, v in kwargs.iteritems():
        if k == 'max_age':
            k = 'max-age'
        morsel[k] = v



web.RequestHandler.set_cookie = set_cookie


def clear_cookie(self, name, path='/', domain=None):
    """Deletes the cookie with the given name."""
    expires = 'Tue, 01 Jun 2000 00:00:00 GMT'
    self.set_cookie(name, value='', path=path, expires=expires, domain=domain)

web.RequestHandler.clear_cookie = clear_cookie


#from model._db import SQLSTORE, mc
from os import getpid
PID = str(getpid()).ljust(7)

#logging.warn("PID:%s", PID)

#@profile_middleware([SQLSTORE, mc])
@profile_middleware([])
def _execute(self, transforms, *args, **kwargs):
    """Executes this request with the given output transforms."""
    request = self.request
    method = request.method
    self._transforms = transforms

    if method not in self.SUPPORTED_METHODS:
        raise HTTPError(405)

#    logging.warn("PID %s %s %s %s%s", PID, self.get_cookie('S') or '-', method, request.host, request.path,)

    if method not in ('GET', 'HEAD') and self.application.settings.get('xsrf_cookies'):
        self.check_xsrf_cookie()
    self.prepare()

    if not self._finished:
        getattr(self, 'init')(*args, **kwargs)
        if not self._finished:
            getattr(self, method.lower())(*args, **kwargs)

            if self._auto_finish and not self._finished:
                self.finish()



    #logging.warn("END %s %s%s", PID, request.host, request.path)

web.RequestHandler._execute = _execute

def _init(self, *args, **kwds):
    pass

web.RequestHandler._execute = _execute
web.RequestHandler.init = _init 


def redirect(self, url, permanent=False):
    """Sends a redirect to the given (optionally relative) URL."""
    if self._headers_written:
        raise Exception('Cannot redirect after headers have been written')
    self.set_status(301 if permanent else 302)
    self.set_header('Location', url)
    self.finish()

web.RequestHandler.redirect = redirect


def xsrf_form_html(self):
    return '<input type="hidden" name="_xsrf" value="%s">'%self.xsrf_token

web.RequestHandler.xsrf_form_html = property(xsrf_form_html)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from _host_user import HOST_USER_CONFIG 
from _env import PREFIX

import re
from mako.template import Template
from os import mkdir
from os.path import join, dirname, exists, isdir

_CONFIG_DIR = join(PREFIX, 'misc/config/nginx')
with open(join(dirname(__file__),"nginx.conf")) as conf:
    tmpl = conf.read()
T = Template(tmpl)


for host, user, config in HOST_USER_CONFIG:
    CONFIG_DIR = join(_CONFIG_DIR, host)
    
    if not exists(CONFIG_DIR):
        mkdir(CONFIG_DIR)
    port_range = range(config.PORT, config.PORT+config.PROCESS_NUM)
    conf = T.render(
        host=config.HOST,
        name=config.HOST.replace(".","_"),
        ports=port_range,
        prefix=PREFIX,
        host_css_js = config.HOST_CSS_JS,
        host_dev_prefix = config.HOST_DEV_PREFIX,
    )

    with open(join(CONFIG_DIR, '%s.conf' % user),'w') as f:
        f.write(conf)


print('created nginx configure files at %s' % _CONFIG_DIR)




# coding:utf-8

from _host_user import HOST_USER_CONFIG
import _env
from os import mkdir
from os.path import abspath, dirname, basename, join, exists

_CONFIG_DIR = join(_env.PREFIX, 'misc/boot','supervisor')

for host, user, o in HOST_USER_CONFIG:
    CONFIG_DIR = join(_CONFIG_DIR, host)

    prjname = basename(_env.PREFIX)
    online = join(_env.PREFIX, 'misc/boot','online.py')

    if not exists(CONFIG_DIR):
        mkdir(CONFIG_DIR)


    port = str(o.PORT)
    procsnum = o.PROCESS_NUM
    srv_name = prjname+'_'+user
    conf = '''[program:%s]
command=sudo -u %s python %s -port=%s%%(process_num)02d
process_name=%%(program_name)s_%%(process_num)02d
directory=.
autorestart=true
redirect_stderr=true
stdout_logfile=/var/log/supervisor/%s-%%(process_num)02d.log
stdout_logfile_maxbytes=1024MB
stdout_logfile_backups=512
stdout_capture_maxbytes=1MB
stdout_events_enabled=false
loglevel=warn
numprocs_start=%s
numprocs=%d

''' % (
        srv_name, 
        user,
        online,  
        port[:-2], 
        srv_name,
        port[-2:], 
        procsnum
)

    f = open(join(CONFIG_DIR, '%s.conf' % user), 'w')
    f.write(conf)
    f.close()

print('created supervisor configure file at %s' % _CONFIG_DIR)




#coding:utf-8
import _env
from misc import config

import sys
from zweb.server.server_tornado_wsgi import Run
#from zweb.server_tornado import Run

from application import application

from zweb.config_loader import CONFIG_LOADED
for i in CONFIG_LOADED:
    print "loaded config : \n\t%s.py"%i.__file__.rsplit(".",1)[0][len(_env.PREFIX)+1:]

if len(sys.argv)!=1 and sys.argv[1][:5] == '-port':
    PORT = sys.argv[1].split('=')[1]
else:
    PORT = config.PORT

run = Run(PORT, application)

if __name__ == "__main__":
    run()





#!/usr/bin/env python
# coding: utf-8

import _env
import time
import subprocess
import sys
from os import makedirs
from os.path import exists , join
from model._db import mysql_config
import envoy
import re

COMM_OPTION = ' -h%s -P%s -u%s -p%s %s '


def backup_table(appname, host, port, name, user, password):
    comm_option = COMM_OPTION % (host, port, user, password, name)

    """
    备份一个表数据的命令实例
    mysqldump --skip-opt --no-create-info 数据库名字 表名 --where="id<2000"
    """
    create_table_option = '--skip-comments --no-data --default-character-set=utf8 --skip-opt --add-drop-table --create-options --quick --hex-blob ' + comm_option

    cmd = 'mysqldump ' + create_table_option
    path = join(_env.PREFIX, 'misc/backup/%s'%appname)

    if not exists(path):
        makedirs(path, 0755)


    r = envoy.run(cmd , timeout=120)
    sql = re.sub( r'(\s+AUTO_INCREMENT\=\d+)', '', r.std_out )
    print path
    with open(join( path, 'mysql.sql'), 'w') as backfile:
        backfile.write(sql)


for appname, item in mysql_config().iteritems():
    host, port, name, user, password = item.get('master').split(':')
    backup_table(appname, host, port, name, user, password)




#coding:utf-8


import hotshot, hotshot.stats
from os import getpid
from cStringIO import StringIO
import os
import re
import time
from os.path import abspath, dirname, join
from tempfile import gettempdir
import sys
tempdir = gettempdir()
PROFILE_LOG_FILENAME_PATTERN = join(tempdir, 'profile-%s.prof')
PROFILE_PICKLE_FILENAME_PATTERN = join(tempdir, 'profile-%s.pickle')
PROFILE_LIMIT = 60

class ProfileInfo:
    """
    Stage 1: profiling
    Stage 2: profile stop, waiting for check result
    """
    def __init__(self, stage, pid, profiler_id=None):
        self.stage = stage
        self.pid = pid
        self.profiler_id = profiler_id and str(profiler_id)

    def get_stats(self):
        stats = load_stats(PROFILE_LOG_FILENAME_PATTERN % self.pid)
        s1 = format_stats(stats, sort='time')
        s2 = format_stats(stats, sort='cumulative')
        return s1 + '\n'*3 + '='*80 + '\n'*3 + s2

def request_start_profile(request, profiler_id, req_count=100):
    request.start_profile = req_count
    pid = getpid()
    info = ProfileInfo(1, pid, profiler_id)
    pickle_fn = PROFILE_PICKLE_FILENAME_PATTERN % pid
    dump(info, open(pickle_fn, 'wb'))

def start_profile(filename=None):
    if filename is None:
        filename = PROFILE_LOG_FILENAME_PATTERN % getpid()
    prof = hotshot.Profile(filename)
    return prof

def on_profile_finished():
    pid = getpid()
    pickle_fn = PROFILE_PICKLE_FILENAME_PATTERN % pid
    try:
        info = load(open(pickle_fn, 'rb'))
        info.stage = 2
    except IOError:
        info = ProfileInfo(2, pid)
    dump(info, open(pickle_fn, 'wb'))

def get_profile_info(pid=None):
    pid = pid or os.getpid()
    pickle_fn = PROFILE_PICKLE_FILENAME_PATTERN % pid
    try:
        profinfo = load(open(pickle_fn, 'rb'))
        return profinfo
    except IOError:
        return None

def delete_profile_info(pid=None):
    pid = pid or os.getpid()
    try:
        os.remove(PROFILE_PICKLE_FILENAME_PATTERN % pid)
    except IOError:
        pass

def load_stats(filename=None):
    if filename is None:
        filename = PROFILE_LOG_FILENAME_PATTERN % getpid()
    stats = hotshot.stats.load(filename)
    try:
        os.remove(filename)
    except IOError:
        pass
    return stats

def format_stats(stats, strip_dirs=None, sort='time'):
    if strip_dirs:
        stats.strip_dirs()
    stats.sort_stats(sort, 'calls')
    so = StringIO()
    stats.stream = so
    stats.print_stats(PROFILE_LIMIT)
    stats.print_callers(PROFILE_LIMIT)
    r = so.getvalue()
    if not strip_dirs:
        rootdir = dirname(
            dirname(abspath(__file__))
        ).lower()+os.sep
        r = r.replace(rootdir, '')
        for i in sys.path:
            if '/site-packages/' in i:
                r = r.replace(i.rsplit('/', 1)[0], '')

    return r

normalize_re = re.compile(r'\d+')
class CallLogger(object):
    """Log every call"""

    def __init__(self, obj):
        self.obj = obj
        self.log = []

    def __getattr__(self, name):
        attr = getattr(self.obj, name)
        def _(*a, **kw):
            call = format_call(name, *a, **kw)
            ncall = normalize_re.sub('-', call)
            t1 = time.time()
            r = attr(*a, **kw)
            cost = time.time() - t1
            self.log.append((call, ncall, cost))
            return r
        return _

def format_call(funcname, *a, **kw):
    arglist = []
    for v in a:
        v = repr(v)
        if len(v) > 70:
            v = v[:70]+'...'
        arglist.append(v)

    r = []
    for k, v in kw.iteritems():
        if len(v) > 70:
            v = v[:70]+'...'
        r.append((k, v))

    arglist += ['%s=%r' % (k, v) for k, v in r]
    return '%s(%s)' % (funcname, ', '.join(arglist))

def profile_middleware(PROFILE_FUNC_LIST):
    def _profile_middleware(execute):
        def _(self, transforms, *args, **kwargs):
            profile = self.get_argument('profile', None)
            profile_cookie = self.get_cookie('profile')
            if profile == '0':
                self.clear_cookie('profile')
                profile = None
                profile_cookie = None
            elif profile is not None:
                self.set_cookie('profile', '1')


            if profile or profile_cookie:

                prof = start_profile()
                for i in PROFILE_FUNC_LIST:
                    i.start_log()

                so = []
                result = None

                cls = self.__class__
                _finish = cls.finish

                def finish(self, chunk=None):
                    if chunk is not None: self.write(chunk)
                    if self._status_code != 200:
                        _finish(self)

                cls.finish = finish
                try:
                    result = prof.runcall(execute, self, transforms, *args, **kwargs)
                    for i in PROFILE_FUNC_LIST:
                        so.append(i.get_log())
                finally:
                    for i in PROFILE_FUNC_LIST:
                        i.stop_log()
                    prof.close()

                    def finish(self, so):
                        _write_buffer = self._write_buffer
                        if ''.join(_write_buffer).lstrip().startswith('<') or profile:
                            #disable output for json
                            stats = load_stats()

                            so.append( format_stats(stats, sort='time') )
                            so.append( format_stats(stats, sort='cumulative') )

                            from cgi import escape
                            splitor = """\n<hr style="border:0;border-bottom:1px dashed;margin:20px 0">\n"""
                            so = splitor.join(escape(i.strip()) for i in so)
                            so = """<pre style="clear:both;padding:1em;background:#ffffde;color:#033;font-size:12px;font-family:fixedsys;white-space: pre-wrap;white-space: -moz-pre-wrap;white-space: -pre-wrap;white-space: -o-pre-wrap;word-wrap:break-word;line-height:18px">\n%s\n</pre>"""%so
                            _write_buffer.append(so)
                        _finish(self)
                    if self._status_code == 200:
                        finish(self, so)
                    cls.finish = _finish

                return result
            return execute(self, transforms, *args, **kwargs)
        return _

    return _profile_middleware




#coding:utf-8
import _tornado
import tornado.ioloop

class Run(object):
    def __init__(self, port, application):
        self.port = port
        self.application = application

    def __call__(self):
        import sys
        if len(sys.argv) > 1 and sys.argv[1].isdigit():
            port = int(sys.argv[1])
        else:
            port = self.port
            if type(port) in (list, tuple):
                port = port[0]

        print 'server on port %s'%port

        self.application.listen(port)
        tornado.ioloop.IOLoop.instance().start()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

def attrcache(f):
    name = f.__name__
    @property
    def _attrcache(self):
        d = self.__dict__
        if name in d:
            return d[name]
        result = f(self)
        d[name] = result
        return result

    return _attrcache




#coding:utf-8

import _env
from _route import route
from _base import View


@route("/")
class index(View):
    def get(self):
        self.render()
#        self.finish("good good study , day day up")

    def post(self):
        mail = self.get_argument('mail','')
        self.render(mail=mail)

if __name__ == "__main__":
    pass

 



import _env
from os.path import join
from zweb.lib.mako.lookup import TemplateLookup
from misc.config import DEBUG 

_PATH = join(_env.PREFIX, 'html')
template_lookup = TemplateLookup(
    directories=_PATH,
    module_directory='/tmp/%s'%_PATH.strip('/').replace('/', '.'),
    disable_unicode=True,
    encoding_errors='ignore',
    default_filters=['str', 'h'],
    filesystem_checks=DEBUG,
    input_encoding='utf-8',
    output_encoding=''
)


def render(htm, **kwds):
    mytemplate = template_lookup.get_template(htm)
    return mytemplate.render(**kwds)



#!/usr/bin/env python
#coding:utf-8
import pyclbr
from os.path import isdir

result = []
def print_route(prefix):
    __import__('%s._site'%prefix)
    mod = __import__(prefix, globals(), locals(), ['_route'], -1)

    prefix = len(prefix) + 1

    for i in sorted(mod._route.route.handlers):
        url, cls = i[:2]
        mn = cls.__module__

        mddict = pyclbr.readmodule(mn)
        cls_name = cls.__name__

        mn = mn.replace('.', '/')
        if isdir(mn):
            mn += '/__init__'
        mn += '.py'

        result.append('\t%s\t%s%s'%(
            ('%s +%s;'%(mn, mddict[cls_name].lineno)).ljust(42),
            url.ljust(42),
            cls_name
        ))



if __name__ == "__main__":
    from misc.boot.site_app import APP
    for site_domain , i in APP:
        for j in i:
            name = j.__name__
            result.append('\n%s\n%s/view/_site.py'%(site_domain, name.replace(".","/")) )
            print_route(name)
    print "\n".join(result) 



# -*- coding: utf-8 -*-

class Route(object):
    def __init__(self):
        self.handlers = []

    def __call__(self, url, **kwds):
        def _(cls):
            self.handlers.append((url, cls, kwds))
            return cls
        return _





#coding:utf-8

import _env

from model._db import Model , ModelMc, redis, redis_key


#REDIS_XXX = redis_key.xxx("%s")


# 记得登记表名字 
# vi _table.py
class Xxx(Model):
    pass

if __name__ == "__main__":
    pass
    for i in Xxx.iter():
        print i 



#!/usr/bin/env python
# coding: utf-8

import _env
import time
import subprocess
import sys
from model._db import mysql_config
from misc.config import BACKUP_PATH

def backup_data(appname, host, port, name, user, password):
    comm_option = COMM_OPTION % (host, port, user, password, name)

    """
    备份一个表数据的命令实例
    mysqldump --skip-opt --no-create-info 数据库名字 表名 --where="id<2000"
    """
    create_table_option = ' --no-create-info --quick --default-character-set=utf8 --skip-opt --hex-blob '+comm_option

    cmd = 'mysqldump ' + create_table_option
    print cmd
    with open(join(BACKUP_PATH, 'data_%s_%s.sql'%(appname,int(time.time()))), 'w') as backfile:
        subprocess.Popen(
            cmd.split(),
            stdout=backfile
        )

def backup_main():
    for appname,item in mysql_config().iteritems():
        host, port, name, user, password = item['master'].split(':')
        backup_data(appname, host, port, name, user, password)

if __name__ == '__main__':
    from os import makedirs
    from os.path import exists
    if not exists(BACKUP_PATH):
        makedirs(BACKUP_PATH,0755)
    backup_main()

print('backup data complete.')




#coding:utf-8
from _env import PREFIX

import default
import getpass
import socket
from misc import config

config.DEBUG = True

from zweb.config_loader import load
load(
    vars(config),
    '_host.%s'%socket.gethostname(),
    '_host.%s_dev'%socket.gethostname(),
    '_user.%s'%getpass.getuser(),
    '_user.%s_dev'%getpass.getuser(),
)




#coding:utf-8
import clint
from clint.textui import puts, indent
from optparse import OptionParser, make_option
from os.path import abspath, dirname, join, exists
from os import symlink, chmod, mkdir
from glob import glob
import shutil
import envoy
import sys
reload(sys)
sys.setdefaultencoding("utf-8")

def _create(template, name):
    from zweb import template as _template
    path = join(dirname(abspath(_template.__file__)), template)
    name = abspath(name)
    if exists(name):
        print "create failed : %s already exist"%name
    else:
        envoy.run('mkdir %s'%name)
        envoy.run("tar zxvf %s.tgz -C %s"%(path, name))
        print 'create success : \n\t%s\n'%name

option_list = [

    make_option(
        '-a', '--app', dest='app', type="string", nargs=1,
        help='在项目中创建一个app'
    ),
    make_option(
        '-p', '--project', 
        dest='project',
        type="string", nargs=1,
        help='创建一个项目'
    ),
]
parser = OptionParser(option_list=option_list)


def main():
    (option, args) = parser.parse_args()

    if option.project:
        name = option.project
        _create("project", name)
        for i in glob("%s/*.sh"%name): 
            chmod(i, 0755)
        
        import socket
        path = "%s/misc/config/_host/%%s.py"%name
        shutil.copy2( path%"_example", path%socket.gethostname())

        import getpass
        path = "%s/misc/config/_user/%%s.py"%name
        shutil.copy2( path%"_example" , path%getpass.getuser())
    
        print "please edit config then run misc/once/install.py\n\ncd %s;vim misc/config/default.py;./misc/once/install.sh\n"%name
    elif option.app:
        if exists(".app"):
            name = option.app
            _create("app", ".app/%s"%name)
            for i in ("model", "css", "js", "html","view","coffee"):
                if not exists(i):
                    mkdir(i)
                path = "%s/%s"%(i,name)
                if not exists(path):
                    symlink("../.app/%s/%s"%(name, i), path)
            for i in ("config","crontab","lib", "once", "backup", "srv"):
                path = "misc/%s/%s"%(i,name)
                if not exists(path):
                    symlink("../../.app/%s/misc/%s"%(name, i), path)
            path = "../../.app/%s/css/_img"%name
            if not exists(path):
                symlink(path , "css/_img/%s"%name)
            print "please add .app/%s to revision control"%name
            print "please modify misc/boot/app_install.py to import your app %s"%name
        else:
            print "no exist dir .app"
    else:
        parser.print_help()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
from cherrypy.wsgiserver import CherryPyWSGIServer
from time import time


def timeit_middleware(func):
    def _(environ, start_response):
        begin = time()
        result = func(environ, start_response)
        logging.info(
            '\n%.2f\t%s //%s%s\n\n------------------------------------------'%(
                    (1000*(time() - begin)),
                    environ.get('REQUEST_METHOD'),
                    environ.get('HTTP_HOST'),
                    environ.get('PATH_INFO'),
            )
        )
        return result
    return _

def WSGIServer(port, application):
    from weberror.evalexception import EvalException
    application = EvalException(application, )
    application = timeit_middleware(application)
    logging.info('\nGAME BEGIN\n\n')
    
    server = CherryPyWSGIServer(('0.0.0.0', port), application, numthreads=2)

    try:
        server.start()
    except KeyboardInterrupt:
        server.stop()





#! /usr/bin/env python
#coding=utf-8

class Errtip(object):
    def __init__(self, template="""<div class="errtip" id="errtip_%s">%s</div>"""):
        self._errtip = {}
        self._template = template

    def __getattr__(self, name):
        if name.startswith('_'):
            return self.__dict__[name]

        if self._errtip and name in self._errtip:
            v = self._errtip[name]
        else:
            v = ''

        html = self._template%(
            name,
            v
        )

        return html

    def __getitem__(self, name):
        return self.__getattr__(name)

    def __nonzero__(self):
        return bool(self._errtip)

    def __setattr__(self, name, value):
        if name.startswith('_'):
            self.__dict__[name] = value
        elif value is not None:
            self.__dict__['_errtip'][name] = value

    def __contains__(self, name):
        return name in self._errtip

if __name__ == '__main__':
    errtip = Errtip()
    print bool(errtip)
    errtip.name = '程序'
    print bool(errtip)
    print errtip.nam2we
    print 'name' in errtip
    print 'x' in errtip



import _env
from collections import defaultdict
import getpass
import socket
from zweb.config_loader import load
from os import mkdir, remove
import sys
from os.path import abspath, join, exists
from shutil import copy2 , rmtree

HOST_USER = defaultdict(set)

#HOST_USER['e1'].add('z32')
HOST_NAME = socket.gethostname()
HOST_USER[HOST_NAME].add(getpass.getuser())

def _host_user_config():
    for user in HOST_USER[HOST_NAME]:

        path = abspath('tmp_config')
        if exists(path):
            rmtree(path)

        mkdir(path)
        sys.path.insert(0, path)
        for from_path , to_path in (
            ('%s/misc/config/default.py'%_env.PREFIX, join(path, 'default.py')) ,
            ('%s/misc/config/_host/%s.py'%(_env.PREFIX, HOST_NAME), join(path, 'host.py')) ,
            ('%s/misc/config/_user/%s.py'%(_env.PREFIX, user), join(path, 'user.py')) ,
        ):
            if exists(from_path):
                if exists(to_path):
                    remove(to_path)
                copy2(from_path, to_path)

        o = load(
            {},
            'default',
            'host' ,
            'user' ,
        )
        sys.path.pop(0)
        yield HOST_NAME, user, o

        rmtree(path)

HOST_USER_CONFIG = list(_host_user_config())

if __name__ == '__main__':
    for host, user, config in host_user_config_iter():
        print host, user , config.HOST




import sys

def _import(name):
    components = name.split('.')

    r = []
    for i in xrange(1,len(components)+1):
        key = ".".join(components[:i])
        if key in sys.modules:
            _mod = sys.modules[key]
            if _mod:
                r.append((key, _mod))
            del sys.modules[key]

    if len(components) == 1:
        mod = __import__(name, globals(), locals(), [], -1)
    else:
        mod = __import__(".".join(components[:-1]), globals(), locals(), [components[-1]], -1)

        for i in components[1:]:
            if hasattr(mod, i):
                mod = getattr(mod, i)
            else:
                mod = None
                break

    for key, _mod in r:
        sys.modules[key] = _mod

    return mod




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import functools
import urllib
import urlparse
from server._tornado import web
import css, js

from misc.config.render import render
from misc.config import HOST 
from model._db import mc
from signal import SIGNAL, Signal

RENDER_KWDS = {
    'css':css,
    'js':js
}

class View(web.RequestHandler):
    def decode_argument(self, value, name=None):
        return value

    def prepare(self):
        mc.reset()

    def on_finish(self):
        Signal.send_delay()
        #print "mc_reset"

    def render(self, template_name=None, **kwds):
        if template_name is None:
            if not hasattr(self, 'template'):
                mod = self.__module__[5:]
                self.template = '/%s/%s.html' % (
                    mod.replace(".","/"),
                    self.__class__.__name__
                )
            template_name = self.template

        current_user = self.current_user
        kwds['current_user'] = current_user
        kwds['current_user_id'] = self.current_user_id
        kwds['request'] = self.request
        kwds['this'] = self
        kwds['_xsrf'] = self._xsrf
        kwds.update(RENDER_KWDS)
        if not self._finished:
            self.finish(render(template_name, **kwds))
    
    @property
    def current_user_id(self):
        if not hasattr(self, '_current_user_id'):
            current_user = self.current_user
            if current_user:
                self._current_user_id = current_user.id
            else:
                self._current_user_id = 0
        return self._current_user_id


    def get_current_user(self):
        return None

    @property
    def _xsrf(self):
        return '_xsrf=%s'%self.xsrf_token


class LoginView(View):
    def prepare(self):
        super(LoginView, self).prepare()
        user_id = self.current_user_id
        if user_id:
            SIGNAL.login_view.send(user_id)
        if not self.current_user:
            _next_redirect(self, "//%s/auth/login"%HOST)


def _next_redirect(self, url):
    if self._finished:
        return
    
    request = self.request
    next_url = request.uri

    if url and next_url and not next_url.startswith(url) and next_url.startswith("/") and len(next_url)>1:
        back = request.protocol + "://" + request.host + request.uri
        url += '?' + urllib.urlencode(dict(next=back))
    self.redirect(url)
    return True


class NoLoginView(View):
    def prepare(self):
        super(NoLoginView, self).prepare()
        if self._finished:
            return
        if self.current_user:
            return self.redirect("/")
    
class JsonLoginView(View):
    def prepare(self):
        super(JsonLoginView, self).prepare()
        if self._finished:
            return
        if not self.current_user:
            self.finish({"login":1})





#!/usr/bin/env python
#coding:utf-8
import sys
from traceback import format_exc
import logging
from tornado.web import HTTPError, httplib

def ErrorMiddleware(application, render, template):
    def _(environ, start_response):
        try:
            return application(environ, start_response)
        except HTTPError, e:
            status_code = e.status_code

            log_message = e.log_message
            start_response(
                '%s %s'%(status_code, httplib.responses[status_code]),
                [('content-type', 'text/html')]
            )
            return [render(
                template,
                status_code=status_code,
                log_message=str(e)
            )]
        except:
            exc_info = sys.exc_info()
            traceback = format_exc()
            start_response(
                '500 Internal Server Error',
                [('content-type', 'text/html')],
                exc_info
            )
            logging.error(environ)
            logging.error(traceback)
            return [render(
                template,
                status_code=500,
                exc_info=exc_info,
                traceback=traceback,
                environ=environ,
            )]
    return _



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# 初始化python的查找路径
import _env
import getpass
import socket
from zweb.config_loader import load

load(
    vars(),
    'default', 
    '_host.%s' % socket.gethostname(),
    '_user.%s' % getpass.getuser(),
)





#!/usr/bin/env python
#coding:utf-8

import _tornado


from misc import config
from misc.config.render import render

from errormiddleware import ErrorMiddleware

import tornado.httpserver
import tornado.ioloop
from tornado.wsgi import WSGIContainer
import _webob_fix


def WSGIServer(port, application):
    application = ErrorMiddleware(application, render, '_error.html')
    container = WSGIContainer(application)
    http_server = tornado.httpserver.HTTPServer(container)
    http_server.listen(port)
    tornado.ioloop.IOLoop.instance().start()



class Run(object):
    def __init__(self, port, application, wsgi_server=WSGIServer):
        self.port = port
        self.application = application
        self.wsgi_server = wsgi_server

    def __call__(self):
        import sys
        if len(sys.argv) > 1 and sys.argv[1].isdigit():
            port = int(sys.argv[1])
        else:
            port = self.port
            if type(port) in (list, tuple):
                port = port[0]

        print 'server on port %s'%port

        self.wsgi_server(port, self.application)





#coding:utf-8
import _env
import misc.config
import misc.config.dev

from zweb.server.server_cherry import WSGIServer
from online import run

run.wsgi_server = WSGIServer




if __name__ == "__main__":
    from zweb.reloader.reload_server import auto_reload
    auto_reload(run)
    





#coding:utf-8
import _env
from os.path import dirname, basename, abspath

MYSQL_DB = basename(dirname(dirname(dirname(abspath(__file__)))))

MYSQL_TABLE = (
   # 用到的表名 , 记得每一行用逗号 , 结尾 
   #'Blog',
)

MYSQL_CONFIG = {
    MYSQL_DB : MYSQL_TABLE
}




#coding:utf-8

from zweb.lib.jsdict import JsDict
from os.path import dirname
import sys
from _import import _import

CONFIG_LOADED = []

def load(self, *args):
    self = JsDict(self)

    prepare_list = [ ]
    finish_list = [ ]

    sys.path.insert(0, dirname(self.__file__))
    global CONFIG_LOADED

    def _load(name):
        #print name
        try:
            mod = _import(name)
        except ImportError:
            return

        if mod is None:
            return

        if mod in CONFIG_LOADED:
            CONFIG_LOADED.pop(CONFIG_LOADED.index(mod))
        CONFIG_LOADED.append(mod)

        mod.__file__.rsplit(".",1)[0]
 
        prepare = getattr(mod, 'pre_config', None)
        if prepare:
            prepare_list.append(prepare)

        finish = getattr(mod, 'post_config', None)
        if finish:
            finish_list.append(finish)

    for i in args:
        _load(i)

    funclist = prepare_list+finish_list
    for _ in funclist:
        _(self)

    sys.path.pop(0)
    return self






import os, sys

def lock(fd):
    import fcntl, errno
    try:
        fcntl.lockf(fd, fcntl.LOCK_EX|fcntl.LOCK_NB)
    except IOError, e:
        if e.errno in (errno.EACCES, errno.EAGAIN):
            print >> sys.stderr, 'There is an instance of', sys.argv[0], 'running. Quit'
            sys.exit(0)
        else:
            raise

def single_process(f):
    lock_file_name = os.path.abspath(sys.argv[0]).lstrip('/').replace('/', '_') + '.lock'
    lock_file_path = os.path.join('/tmp/', lock_file_name)
    def _(*a, **kw):
        fd = os.open(lock_file_path, os.O_CREAT|os.O_RDWR, 0660)
        try:
            lock(fd)
            return f(*a, **kw)
        finally:
            os.close(fd)
    return _



#coding:utf-8
import _env
from misc.config import HOST


APP_INSTALL = [] 

#from view import blog
#APP_INSTALL.append(
#    (
#        HOST,
#        (
#            blog, 
#        )
#    )
#)
#
#from view import vps
#APP_INSTALL.append(
#    (
#        "vps.%s"%HOST,
#        (
#            vps, 
#        )
#    )
#)
#
#from view import user
#APP_INSTALL.append(
#   (
#       ".*",
#       (app.user)
#   )
#)



#coding:utf-8

import _env
from os.path import join, dirname, abspath, exists
from os import walk, mkdir, remove, makedirs
from collections import defaultdict
from hashlib import md5
from glob import glob
from base64 import urlsafe_b64encode
import envoy
import zweb
import os
from tempfile import mktemp
from misc.config import HOST_CSS_JS, HOST_DEV_PREFIX

BULID = join(_env.PREFIX, '.build')
BULID_EXIST = set(glob(BULID+'/*'))
PATH2HASH = {}
if not exists(BULID):
    mkdir(BULID)

def dirwalk(dirname):
    base = join(_env.PREFIX, dirname)
    merge = []
    file = []
    suffix = '.%s'%dirname
    for dirpath, dirnames, filenames in walk(base, followlinks=True):
        for i in filenames:
            path = abspath(join(dirpath, i))
            if i == 'merge.conf':
                merge.append((path, merge_conf(path, base)))
            if i.endswith(suffix):
                file.append(path)
    return file , merge


def merge_conf(file, base):
    ft = defaultdict(list)
    p = None
    dirpath = dirname(file)
    with open(file) as f:
        for line in f:
            line = line.strip()
            if line.startswith("#") or not line:
                continue
            if line[0] == '/':
                path = base+line
            else:
                path = join(dirpath, line)
            if line.endswith(':'):
                p = path[:-1].strip()
            elif line and p:
                ft[p].append(path)
    return ft

#@import url(ctrl/main.css);
#@import url(ctrl/zsite.css);
#@import url(ctrl/feed.css);


def merge_css( src_list):
    result = [ ]
    for i in src_list:
        result.append("""@import url(%s);"""%(i[len(_env.PREFIX)+4:]))
    return result

def merge_js( src_list):
    result = [
        '''function LOAD(js){ document.write('<script src="'+js+'"></'+"script>") }'''
    ]

    for i in src_list:
        result.append("""LOAD('http://%sjs%s%s')"""%(HOST_DEV_PREFIX, HOST_CSS_JS, i[len(_env.PREFIX)+3:]))
    return result


def run(suffix):
    file_list , merge_list = dirwalk(suffix)
    file_set = set(file_list)

    to_merge = defaultdict(list)
    for merge_conf, merge in merge_list:
        for to_file, src_list in merge.iteritems():
            if to_file in file_set:
                file_set.remove(to_file)
            for i in src_list:
                if exists(i):
                    to_merge[to_file].append(i)
                else:
                    print merge_conf , 'ERROR'
                    print '\t', i , 'NOT EXIST'

    if suffix == 'css':
        merger = merge_css
        cmd = 'java -jar %s --charset=utf-8 --type css  -o %%s %%s'% join(dirname(abspath(zweb.__file__)), 'utils/yuicompressor.jar')
    else:
        merger = merge_js
        cmd = 'uglifyjs -nc -o %s %s '

    for i in file_set:
        base = join(_env.PREFIX, suffix)
        with open(i) as infile:
            hash = hash_name(infile.read(), i)
            path = join(BULID, hash)+"."+suffix
            if path not in BULID_EXIST:
                t = cmd%(path, i)
                print t
                envoy.run(t)


    for to_file, src_list, in to_merge.iteritems():
        
        dirpath = dirname(to_file)
        if not exists(dirpath):
            makedirs(dirpath)

        r = merger( src_list)
        with open(to_file, 'w') as to:
            r = '\n'.join(r)
            to.write(r)

        r = []
        for i in src_list:
            with open(join(BULID, PATH2HASH[i])) as t:
                r.append(t.read())
        r = '\n'.join(r)
        hash = hash_name(r, to_file)
        path = join(BULID, hash)+"."+suffix
        #print path
        if path not in BULID_EXIST:
            tmp = mktemp()
            with open(tmp, 'w') as f:
                f.write(r)

            t = cmd%(path, tmp)
            print t
            envoy.run(t)


def hash_name(content, path):
    hash = urlsafe_b64encode(md5(content).digest()).rstrip('=')
    PATH2HASH[path] = hash+"."+path.rsplit(".",1)[-1]
    return hash

run('css')
run('js')

for i in BULID_EXIST-set(BULID+'/'+i for i in PATH2HASH.itervalues()):
    if i.endswith(".css") or i.endswith(".js"):
        print 'remove', i
        remove(i)

init = defaultdict(list)
for file_name, hash in  PATH2HASH.iteritems():
    dirname, file_name = file_name[len(_env.PREFIX)+1:].split('/', 1)
    init[dirname].append(( file_name, hash ))

for suffix, flist in init.iteritems():
    with open(join(_env.PREFIX, suffix, '_hash_.py'), 'w') as h:
        h.write("""#coding:utf-8\n
import _env

__HASH__ =  {
""")
        for name, hash in flist:
            h.write(
                """    "%s" : '%s', #%s\n"""%(
                    name,
                    hash,
                    name.rsplit('.', 1)[0].replace('.', '_').replace('-', '_').replace('/', '_')
                )
            )
        h.write('}')

        h.write("""


from misc.config import DEBUG, HOST_CSS_JS, HOST_DEV_PREFIX
from os.path import dirname,basename
__vars__ = vars()

for file_name, hash in __HASH__.iteritems():
    
    if DEBUG:
        value = "http://%s%s%s/%s"%(HOST_DEV_PREFIX, basename(dirname(__file__)), HOST_CSS_JS,  file_name)
    else:
        value = "http://%s/%s"%(HOST_CSS_JS, hash) 
    
    name = file_name.rsplit('.', 1)[0].replace('.', '_').replace('-', '_').replace('/', '_')
    
    __vars__[name] = value
                            
""")






#!/usr/bin/env python
# -*- coding: utf-8 -*-
from os.path import abspath, dirname, basename, join, exists
from os import walk

FROM_STRING = """
from misc from misc
"""

TO_STRING = """
from misc
"""

def run():
    from_string = FROM_STRING.strip()
    to_string = TO_STRING.strip()
    for from_s, to_s in zip(filter(bool, FROM_STRING.split('\n')), filter(bool, TO_STRING.split('\n'))):
        replace(from_s.strip(), to_s.strip())

def replace(from_string, to_string):
    from_string = from_string.strip()
    to_string = to_string.strip()

    file = abspath(__file__)

    for dirpath, dirnames, filenames in walk(dirname(file)):
        dirbase = basename(dirpath)
        if dirbase.startswith('.'):
            continue

        for filename in filenames:
            suffix = filename.rsplit('.', 1)[-1]
            if suffix not in ('py', 'htm', 'txt', 'conf', 'css', 'h', 'template', 'js', 'html',"rst"):
                continue
            path = join(dirpath, filename)
            if path == file:
                continue
            if not exists(path):
                continue
            with open(path) as f:
                content = f.read()
            t = content.replace(from_string, to_string)
            if t != content:
                with open(path, 'wb') as f:
                    f.write(t)

run()




# -*- coding: utf-8 -*-
import _env
import tornado.wsgi
from misc.config import DEBUG

application = tornado.wsgi.WSGIApplication(
    login_url='/auth/login' ,
    xsrf_cookies=True,
    debug=DEBUG
)

from app_install import APP_INSTALL 
from zweb.app_install import app_install
app_install(application, APP_INSTALL)
 



#coding:utf-8
import traceback

def _route_chain(view, args):
    handlers = []
    for i in args:
        prefix = i.__name__

        #import view.blog._install
        try:
            __import__('%s.%s'%(prefix,view))
        except ImportError, e:
            traceback.print_exc() 
            raise e
        i = __import__(prefix, globals(), locals(), ['_route'], -1)
        handlers.extend(i._route.route.handlers)

    return handlers

def app_install(application, view, app_list):
    for domain , route_list in app_list:
        if not domain.startswith("."):
            domain = domain.replace('.', r"\.")
        application.add_handlers(
            domain,
            _route_chain(view, route_list)
        )




#coding:utf-8

from misc.config import HOST
from types import FunctionType
from model._db import celery
from model._celery import SIGNAL_IMPORT
import atexit


def call(func, args):
    if type(func) is FunctionType:
        args = args[:func.func_code.co_argcount]
    func(*args)

class Signal(object):
    _DELAY = []
    
    def __init__(self, name):
        self._sync = []
        self._async = []
        self.name = name

        @celery.task(name="%s.%s"%(HOST,name))
        def _(*args):
            for func in self._async:
                call(func,args)

        self._delay = _.delay  

    # SINGAL.xxxxxx.rm(func)
    def rm(self, func):
        if func in self._sync:
            self._sync.remove(func) 
        if func in self._async:
            self._async.remove(func) 

    @classmethod
    def send_delay(cls):
        for func, args in cls._DELAY: 
            func(*args)
        cls._DELAY = []

    def send(self, *args):
        name = self.name
        if name in SIGNAL_IMPORT:
            for i in SIGNAL_IMPORT[name]:
                __import__(i)
            del SIGNAL_IMPORT[name]

        for func in self._sync:
            call(func,args)

        if self._async:
            self._DELAY.append((self._delay, args)) 

    def __call__(self, func):
        self._sync.append(func)
        return func

    def delay(self, func):
        self._async.append(func)
        return func

atexit.register(Signal.send_delay)


class _(object):
    def __getattr__(self, name):
        d = self.__dict__
        if name not in d:
            d[name] = Signal(name)
        return d[name]

        

SIGNAL = _()

if __name__ == "__main__":


    @SIGNAL.follow_new
    def _follow_new(a,b):
        print a,b 
   
    @SIGNAL.follow_new.delay
    def _follow_new(a, b):
        print a,b
    SIGNAL.follow_new.send(1 , 3)
    SIGNAL.send_delay()




import webob.request

def readinto(self, buff):
    if not self.remaining:
        return 0
    sz0 = min(len(buff), self.remaining)
    data = self.file.read(sz0)
    sz = len(data)
    self.remaining -= sz
    #if not data:
    if sz < sz0 and self.remaining:
        pass
        #raise DisconnectionError(
        #    "The client disconnected while sending the POST/PUT body "
        #    + "(%d more bytes were expected)" % self.remaining
        #)
    buff[:sz] = data
    return sz

webob.request.LimitedLengthFile.readinto = readinto



#coding:utf-8
import _env
from misc import config
import zorm.config as zorm_config

from misc.config import DEBUG
zorm_config.DEBUG = DEBUG


from redis import StrictRedis
from misc.config import REDIS_CONFIG

zorm_config.redis = redis = StrictRedis(**REDIS_CONFIG)



from zorm.mc_connection import init_mc
from misc.config import MEMCACHED_ADDR , DISABLE_LOCAL_CACHED

mc = init_mc(
    memcached_addr=MEMCACHED_ADDR,
    disable_local_cached=DISABLE_LOCAL_CACHED
)

zorm_config.mc = mc

from zorm import sqlstore

def mysql_config():
    from glob import glob
    from misc.config import MYSQL_HOST , MYSQL_PORT , MYSQL_USER , MYSQL_PASSWD , MYSQL_PREFIX
    from os.path import dirname, abspath, join, exists, basename
    import sys
    sqlfarm = {}
    for i in glob('%s/.app/*'%_env.PREFIX):
        if '.' in basename(i):
            continue
        path = join(i, 'misc/config/_table.py')
        if exists(path):
            if '__init__' in sys.modules:
                del sys.modules['__init__']
            if '_table' in sys.modules:
                del sys.modules['_table']
            d = dirname(path)
            sys.path.insert(0, d)
            c = __import__('__init__')
            sys.path.pop(0)

            for mysql_db, mysql_table in c.MYSQL_CONFIG.iteritems():
                db = '%s_%s'%(MYSQL_PREFIX, mysql_db)

                sqlfarm[basename(i)] = {
                    'master': '%s:%s:%s:%s:%s' % ( MYSQL_HOST, MYSQL_PORT, db, MYSQL_USER, MYSQL_PASSWD),
                    'tables': mysql_table,
                }

    return sqlfarm

_sqlstore = sqlstore.SqlStore(db_config=mysql_config(), charset='utf8')

def db_by_table(table):
    return _sqlstore.get_db_by_table(table)

zorm_config.db_by_table = db_by_table



from zorm.model import Model, ModelMc
from zorm.redis import redis, redis_key




if __name__ == '__main__':
    pass
    print mysql_config()




#coding:utf-8

from setuptools import setup, find_packages 

setup(
    name='zweb',
    version="0.4300",
    description="A web site framework",
    author="zuroc 张沈鹏",
    author_email="zsp042@gmail.com",
    packages = ['zweb'],
    zip_safe=False,
    include_package_data=True,
    install_requires = [
        'zorm>=0.03',
        'tornado>=2.3',
        'mako',
        'clint',
        'cherrypy>=3.2.2',
        'weberror>=0.10.3',
        'envoy',
        'supervisor',
    ],
    entry_points = {
        'console_scripts': [
            'zweb=zweb.script:main',
        ],
    },
)

